/*
** Module   : Operators.cs
** Abstract : Delegates the CLR decorated methods to overloaded operators from assembly converted
** from java p2jpl.
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**           Golden Code Development Corporation
**                      CONFIDENTIAL
**
** -#- -I- --Date-- -------------------------------- Description --------------------------------
** 001 OM  20140313 Created initial version with partial support of existing methods from
**                  Operators.java at this time (only the overloaded equality operator).
*/

using System;

/**
 * The role of this class is to resolve the SQL Server method overload constraint and simplify
 * the name of the functions.
 *
 * Each method of this class has as correspondent method in the original
 * <code>com.goldencode.p2j.persist.pl.Operators</code> that will call with the parameters
 * converted for wrapped in IKVM converted datatypes. The method names of the
 * original class is decorated with as list of characters, one for each parameter, each such
 * character represents the type of the parameters in the original method signature. This way,
 * there are no overloaded methods so SQL Server cannot complain about this.
 *
 * The decorations are as follows:
 * <ul>
 *    <li>s - for character / String / (string / String) / NVARCHAR(4000)
 *    <li>b - for logical / (boolean / Boolean) / (bool / Boolean) / BIT
 *    <li>i - for integer / (int / Integer) / (int / Int32) / INT
 *    <li>l - for int64 / (long / Long) / (long / Int64) / BIGINT
 *    <li>r - for decimal / (double / BigDecimal) / (double / Double) / (DOUBLE PRECISION / DECIMAL(38))
 *    <li>d - for date / Date / DateTime / DATE
 *    <li>t - for datetime / Timestamp / DateTime / DATETIME2
 * </ul>
 *
 * For each function an UDF will be defined in SQL having the same name as the method of this
 * class.
 *
 * Note that, if the original interface of the <code>Operators</code> is changed (new methods are
 * added or existing ones change their signature) the current class must be adjusted to fit those
 * changes (add/remove method or signature change).
 */
public class Operators
{
    /**
     * Private constructor. This class is never instantiated. All access is done using static
     * public methods.
     */
    private Operators()
    {
    }

   /**
    * Test two values for equality.
    *
    * @see  com.goldencode.p2j.util.CompareOps#isEqual(
    *          com.goldencode.p2j.util.BaseDataType,
    *          com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean eq_bb(Boolean a, Boolean b)
   {
       return com.goldencode.p2j.persist.pl.Operators.eq(
           java.lang.Boolean.valueOf(a),
           java.lang.Boolean.valueOf(b)
       ).booleanValue();
   }

   /**
    * Test two values for equality.
    *
    * @see  com.goldencode.p2j.util.CompareOps#isEqual(
    *          com.goldencode.p2j.util.BaseDataType,
    *          com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean eq_rr(Double a, Double b)
   {
       return com.goldencode.p2j.persist.pl.Operators.eq(
           java.math.BigDecimal.valueOf(a),
           java.math.BigDecimal.valueOf(b)
       ).booleanValue();
   }

   /**
    * Test two values for equality.
    *
    * @see  com.goldencode.p2j.util.CompareOps#isEqual(
    *          com.goldencode.p2j.util.BaseDataType,
    *          com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean eq_ri(Double a, Int32 b)
   {
       return com.goldencode.p2j.persist.pl.Operators.eq(
           java.math.BigDecimal.valueOf(a),
           java.lang.Integer.valueOf(b)
       ).booleanValue();
   }

   /**
    * Test two values for equality.
    *
    * @see  com.goldencode.p2j.util.CompareOps#isEqual(
    *          com.goldencode.p2j.util.BaseDataType,
    *          com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean eq_rl(Double a, Int64 b)
   {
       return com.goldencode.p2j.persist.pl.Operators.eq(
           java.math.BigDecimal.valueOf(a),
           java.lang.Long.valueOf(b)
       ).booleanValue();
   }

   /**
    * Test two values for equality.
    *
    * @see  com.goldencode.p2j.util.CompareOps#isEqual(
    *          com.goldencode.p2j.util.BaseDataType,
    *          com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean eq_dd(DateTime a, DateTime b)
   {
       return com.goldencode.p2j.persist.pl.Operators.eq(
           Common.toDate(a),
           Common.toDate(b)
       ).booleanValue();
   }

   /**
    * Test two values for equality.
    *
    * @see  com.goldencode.p2j.util.CompareOps#isEqual(
    *          com.goldencode.p2j.util.BaseDataType,
    *          com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean eq_ir(Int32 a, Double b)
   {
       return com.goldencode.p2j.persist.pl.Operators.eq(
           java.lang.Integer.valueOf(a),
           java.math.BigDecimal.valueOf(b)
       ).booleanValue();
   }

   /**
    * Test two values for equality.
    *
    * @see  com.goldencode.p2j.util.CompareOps#isEqual(
    *          com.goldencode.p2j.util.BaseDataType,
    *          com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean eq_ii(Int32 a, Int32 b)
   {
       return com.goldencode.p2j.persist.pl.Operators.eq(
           java.lang.Integer.valueOf(a),
           java.lang.Integer.valueOf(b)
       ).booleanValue();
   }

   /**
    * Test two values for equality.
    *
    * @see  com.goldencode.p2j.util.CompareOps#isEqual(
    *          com.goldencode.p2j.util.BaseDataType,
    *          com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean eq_il(Int32 a, Int64 b)
   {
       return com.goldencode.p2j.persist.pl.Operators.eq(
           java.lang.Integer.valueOf(a),
           java.lang.Long.valueOf(b)
       ).booleanValue();
   }

   /**
    * Test two values for equality.
    *
    * @see  com.goldencode.p2j.util.CompareOps#isEqual(
    *          com.goldencode.p2j.util.BaseDataType,
    *          com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean eq_lr(Int64 a, Double b)
   {
       return com.goldencode.p2j.persist.pl.Operators.eq(
           java.lang.Long.valueOf(a),
           java.math.BigDecimal.valueOf(b)
       ).booleanValue();
   }

   /**
    * Test two values for equality.
    *
    * @see  com.goldencode.p2j.util.CompareOps#isEqual(
    *          com.goldencode.p2j.util.BaseDataType,
    *          com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean eq_li(Int64 a, Int32 b)
   {
       return com.goldencode.p2j.persist.pl.Operators.eq(
           java.lang.Long.valueOf(a),
           java.lang.Integer.valueOf(b)
       ).booleanValue();
   }

   /**
    * Test two values for equality.
    *
    * @see  com.goldencode.p2j.util.CompareOps#isEqual(
    *          com.goldencode.p2j.util.BaseDataType,
    *          com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean eq_ll(Int64 a, Int64 b)
   {
       return com.goldencode.p2j.persist.pl.Operators.eq(
           java.lang.Long.valueOf(a),
           java.lang.Long.valueOf(b)
       ).booleanValue();
   }

   /**
    * Test two values for equality.
    *
    * @see  com.goldencode.p2j.util.CompareOps#isEqual(
    *          com.goldencode.p2j.util.BaseDataType,
    *          com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean eq_ss(string a, string b)
   {
       return com.goldencode.p2j.persist.pl.Operators.eq(a, b).booleanValue();
   }

   /**
    * Test two values for equality.
    *
    * @see  com.goldencode.p2j.util.CompareOps#isEqual(
    *          com.goldencode.p2j.util.BaseDataType,
    *          com.goldencode.p2j.util.BaseDataType)
    */
   public static Boolean eq_tt(DateTime a, DateTime b)
   {
       return com.goldencode.p2j.persist.pl.Operators.eq(
           Common.toTimestamp(a),
           Common.toTimestamp(b)
       ).booleanValue();
   }
}
